@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 """Misc. xonsh tools.
 
 The following implementations were forked from the IPython project:
@@ -17,35 +16,35 @@ Implementations:
 * indent()
 
 """
-import builtins
+import ast
 import collections
 import collections.abc as cabc
 import contextlib
 import ctypes
 import datetime
-from distutils.version import LooseVersion
 import functools
 import glob
 import itertools
+import operator
 import os
 import pathlib
 import re
+import shlex
+import string
 import subprocess
 import sys
 import threading
 import traceback
+import typing as tp
 import warnings
-import operator
-import ast
-import string
 
 # adding imports from further xonsh modules is discouraged to avoid circular
 # dependencies
 from xonsh import __version__
-from xonsh.lazyasd import LazyObject, LazyDict, lazyobject
+from xonsh.lazyasd import LazyDict, LazyObject, lazyobject
 from xonsh.platform import (
-    scandir,
     DEFAULT_ENCODING,
+    HAS_PYGMENTS,
     ON_LINUX,
     ON_WINDOWS,
     expanduser,
@@ -63,6 +62,13 @@ def is_superuser():
     return rtn
 
 
+@lazyobject
+def xsh():
+    from xonsh.built_ins import XSH
+
+    return XSH
+
+
 class XonshError(Exception):
     pass
 
@@ -77,7 +83,7 @@ class XonshCalledProcessError(XonshError, subprocess.CalledProcessError):
     returncode of the command is nonzero.
 
     Example:
-    -------
+    --------
         try:
             for line in !(ls):
                 print(line)
@@ -99,8 +105,7 @@ class XonshCalledProcessError(XonshError, subprocess.CalledProcessError):
 def expand_path(s, expand_user=True):
     """Takes a string path and expands ~ to home if expand_user is set
     and environment vars if EXPAND_ENV_VARS is set."""
-    session = getattr(builtins, "__xonsh__", None)
-    env = os_environ if session is None else getattr(session, "env", os_environ)
+    env = xsh.env or os_environ
     if env.get("EXPAND_ENV_VARS", False):
         s = expandvars(s)
     if expand_user:
@@ -121,18 +126,24 @@ def _expandpath(path):
     """Performs environment variable / user expansion on a given path
     if EXPAND_ENV_VARS is set.
     """
-    session = getattr(builtins, "__xonsh__", None)
-    env = os_environ if session is None else getattr(session, "env", os_environ)
+    env = xsh.env or os_environ
     expand_user = env.get("EXPAND_ENV_VARS", False)
     return expand_path(path, expand_user=expand_user)
 
 
+def simple_random_choice(lst):
+    """Returns random element from the list with length less than 1 million elements."""
+    size = len(lst)
+    if size > 1000000:  # microsecond maximum
+        raise ValueError("The list is too long.")
+    return lst[datetime.datetime.now().microsecond % size]
+
+
 def decode_bytes(b):
     """Tries to decode the bytes using XONSH_ENCODING if available,
     otherwise using sys.getdefaultencoding().
     """
-    session = getattr(builtins, "__xonsh__", None)
-    env = os_environ if session is None else getattr(session, "env", os_environ)
+    env = xsh.env or os_environ
     enc = env.get("XONSH_ENCODING") or DEFAULT_ENCODING
     err = env.get("XONSH_ENCODING_ERRORS") or "strict"
     return b.decode(encoding=enc, errors=err)
@@ -224,7 +235,7 @@ class EnvPath(cabc.MutableSequence):
         return all(map(operator.eq, self, other))
 
     def _repr_pretty_(self, p, cycle):
-        """ Pretty print path list """
+        """Pretty print path list"""
         if cycle:
             p.text("EnvPath(...)")
         else:
@@ -269,22 +280,40 @@ class EnvPath(cabc.MutableSequence):
         None
 
         """
+        data = str(expand_path(data))
         if data not in self._l:
             self._l.insert(0 if front else len(self._l), data)
         elif replace:
-            self._l.remove(data)
+            # https://stackoverflow.com/a/25251306/1621381
+            self._l = list(filter(lambda x: x != data, self._l))
             self._l.insert(0 if front else len(self._l), data)
 
 
+class FlexibleFormatter(string.Formatter):
+    """Support nested fields inside conditional formatters
+
+    e.g. template ``{user:| {RED}{}{RESET}}`` will become ``| {RED}user{RESET}`` when user=user.
+    """
+
+    def get_value(self, key: "int|str", args, kwargs) -> str:
+        if isinstance(key, int):
+            return args[key]
+        else:
+            if key in kwargs:
+                return kwargs[key]
+            # in case of colors, this will work without nested braces
+            return "{" + key + "}"
+
+
 @lazyobject
 def FORMATTER():
-    return string.Formatter()
+    return FlexibleFormatter()
 
 
-class DefaultNotGivenType(object):
+class DefaultNotGivenType:
     """Singleton for representing when no default value is given."""
 
-    __inst = None
+    __inst: tp.Optional["DefaultNotGivenType"] = None
 
     def __new__(cls):
         if DefaultNotGivenType.__inst is None:
@@ -324,7 +353,7 @@ def balanced_parens(line, mincol=0, maxcol=None, lexer=None):
     """Determines if parentheses are balanced in an expression."""
     line = line[mincol:maxcol]
     if lexer is None:
-        lexer = builtins.__xonsh__.execer.parser.lexer
+        lexer = xsh.execer.parser.lexer
     if "(" not in line and ")" not in line:
         return True
     cnt = 0
@@ -339,6 +368,15 @@ def balanced_parens(line, mincol=0, maxcol=None, lexer=None):
     return cnt == 0
 
 
+def ends_with_colon_token(line, lexer=None):
+    """Determines whether a line ends with a colon token, ignoring comments."""
+    if lexer is None:
+        lexer = xsh.execer.parser.lexer
+    lexer.input(line)
+    toks = list(lexer)
+    return len(toks) > 0 and toks[-1].type == "COLON"
+
+
 def find_next_break(line, mincol=0, lexer=None):
     """Returns the column number of the next logical break in subproc mode.
     This function may be useful in finding the maxcol argument of
@@ -347,7 +385,7 @@ def find_next_break(line, mincol=0, lexer=None):
     if mincol >= 1:
         line = line[mincol:]
     if lexer is None:
-        lexer = builtins.__xonsh__.execer.parser.lexer
+        lexer = xsh.execer.parser.lexer
     if RE_END_TOKS.search(line) is None:
         return None
     maxcol = None
@@ -385,7 +423,7 @@ def subproc_toks(
     normal parentheses. Greedy is False by default.
     """
     if lexer is None:
-        lexer = builtins.__xonsh__.execer.parser.lexer
+        lexer = xsh.execer.parser.lexer
     if maxcol is None:
         maxcol = len(line) + 1
     lexer.reset()
@@ -520,14 +558,12 @@ def get_line_continuation():
     mode on Windows the backslash must be preceded by a space. This is because
     paths on Windows may end in a backslash.
     """
-    if (
-        ON_WINDOWS
-        and hasattr(builtins.__xonsh__, "env")
-        and builtins.__xonsh__.env.get("XONSH_INTERACTIVE", False)
-    ):
-        return " \\"
-    else:
-        return "\\"
+    if ON_WINDOWS:
+        env = getattr(xsh, "env", None) or {}
+        if env.get("XONSH_INTERACTIVE", False):
+            return " \\"
+
+    return "\\"
 
 
 def get_logical_line(lines, idx):
@@ -609,7 +645,7 @@ def subexpr_before_unbalanced(expr, ltok, rtok):
     subexpr, _, post = expr.rpartition(ltok)
     nrtoks_in_post = post.count(rtok)
     while nrtoks_in_post != 0:
-        for i in range(nrtoks_in_post):
+        for _ in range(nrtoks_in_post):
             subexpr, _, post = subexpr.rpartition(ltok)
         nrtoks_in_post = post.count(rtok)
     _, _, subexpr = subexpr.rpartition(rtok)
@@ -650,7 +686,7 @@ def safe_hasattr(obj, attr):
     try:
         getattr(obj, attr)
         return True
-    except Exception:  # pylint:disable=bare-except
+    except Exception:
         return False
 
 
@@ -695,7 +731,7 @@ def get_sep():
     """Returns the appropriate filepath separator char depending on OS and
     xonsh options set
     """
-    if ON_WINDOWS and builtins.__xonsh__.env.get("FORCE_POSIX_PATHS"):
+    if ON_WINDOWS and xsh.env.get("FORCE_POSIX_PATHS"):
         return os.altsep
     else:
         return os.sep
@@ -718,8 +754,7 @@ def fallback(cond, backup):
 # See the Python software license: https://docs.python.org/3/license.html
 # Copyright (c) Python Software Foundation. All rights reserved.
 class _RedirectStream:
-
-    _stream = None
+    _stream: tp.Optional[str] = None
 
     def __init__(self, new_target):
         self._new_target = new_target
@@ -763,7 +798,7 @@ def _yield_accessible_unix_file_names(path):
     """yield file names of executable files in path."""
     if not os.path.exists(path):
         return
-    for file_ in scandir(path):
+    for file_ in os.scandir(path):
         try:
             if file_.is_file() and os.access(file_.path, os.X_OK):
                 yield file_.name
@@ -782,9 +817,9 @@ def _executables_in_posix(path):
 def _executables_in_windows(path):
     if not os.path.isdir(path):
         return
-    extensions = builtins.__xonsh__.env["PATHEXT"]
+    extensions = xsh.env["PATHEXT"]
     try:
-        for x in scandir(path):
+        for x in os.scandir(path):
             try:
                 is_file = x.is_file()
             except OSError:
@@ -804,7 +839,7 @@ def _executables_in_windows(path):
 
 
 def executables_in(path):
-    """Returns a generator of files in path that the user could execute. """
+    """Returns a generator of files in path that the user could execute."""
     if ON_WINDOWS:
         func = _executables_in_windows
     else:
@@ -822,7 +857,7 @@ def debian_command_not_found(cmd):
     if not ON_LINUX:
         return ""
 
-    cnf = builtins.__xonsh__.commands_cache.lazyget(
+    cnf = xsh.commands_cache.lazyget(
         "command-not-found", ("/usr/lib/command-not-found",)
     )[0]
 
@@ -831,8 +866,8 @@ def debian_command_not_found(cmd):
 
     c = "{0} {1}; exit 0"
     s = subprocess.check_output(
-        c.format(cnf, cmd),
-        universal_newlines=True,
+        c.format(cnf, shlex.quote(cmd)),
+        text=True,
         stderr=subprocess.STDOUT,
         shell=True,
     )
@@ -867,7 +902,8 @@ def command_not_found(cmd, env):
     return rtn
 
 
-def suggest_commands(cmd, env, aliases):
+@functools.lru_cache
+def suggest_commands(cmd, env):
     """Suggests alternative commands given an environment and aliases."""
     if not env.get("SUGGEST_COMMANDS"):
         return ""
@@ -878,15 +914,15 @@ def suggest_commands(cmd, env, aliases):
     cmd = cmd.lower()
     suggested = {}
 
-    for alias in builtins.aliases:
+    for alias in xsh.aliases:
         if alias not in suggested:
             if levenshtein(alias.lower(), cmd, thresh) < thresh:
                 suggested[alias] = "Alias"
 
-    for _cmd in builtins.__xonsh__.commands_cache.all_commands:
+    for _cmd in xsh.commands_cache.all_commands:
         if _cmd not in suggested:
             if levenshtein(_cmd.lower(), cmd, thresh) < thresh:
-                suggested[_cmd] = "Command ({0})".format(_cmd)
+                suggested[_cmd] = f"Command ({_cmd})"
 
     suggested = collections.OrderedDict(
         sorted(
@@ -899,13 +935,13 @@ def suggest_commands(cmd, env, aliases):
         rtn = command_not_found(cmd, env)
     else:
         oneof = "" if num == 1 else "one of "
-        tips = "Did you mean {}the following?".format(oneof)
+        tips = f"Did you mean {oneof}the following?"
         items = list(suggested.popitem(False) for _ in range(num))
         length = max(len(key) for key, _ in items) + 2
         alternatives = "\n".join(
             "    {: <{}} {}".format(key + ":", length, val) for key, val in items
         )
-        rtn = "{}\n{}".format(tips, alternatives)
+        rtn = f"{tips}\n{alternatives}"
         c = command_not_found(cmd, env)
         rtn += ("\n\n" + c) if len(c) > 0 else ""
     return rtn
@@ -913,7 +949,7 @@ def suggest_commands(cmd, env, aliases):
 
 def _get_manual_env_var(name, default=None):
     """Returns if the given variable is manually set as well as it's value."""
-    env = getattr(builtins.__xonsh__, "env", None)
+    env = getattr(xsh, "env", None)
     if env is None:
         env = os_environ
         manually_set = name in env
@@ -960,8 +996,39 @@ def print_warning(msg):
     sys.stderr.write(msg)
 
 
-def print_exception(msg=None):
-    """Print exceptions with/without traceback."""
+def print_exception(msg=None, exc_info=None):
+    """Print given exception (or current if None) with/without traceback and set sys.last_type, sys.last_value, sys.last_traceback accordingly."""
+
+    # is no exec_info() triple is given, use the exception beeing handled at the moment
+    if exc_info is None:
+        exc_info = sys.exc_info()
+
+    # these values (initialized with their default for traceback.print_exception) control how an exception is printed
+    limit = None
+    chain = True
+
+    _, debug_level = _get_manual_env_var("XONSH_DEBUG", 0)
+
+    # the interal state of the parsers stack is
+    # not helpful in normal operation (XONSH_DEBUG == 0).
+    # this is also done to be consistent with python
+    is_syntax_error = issubclass(exc_info[0], SyntaxError)
+
+    # XonshErrors don't show where in the users code they occured
+    # (most are reported deeper in the callstack, e.g. see procs/pipelines.py),
+    # but only show non-helpful xonsh internals.
+    # These are only relevent when developing/debugging xonsh itself.
+    # Therefore, dont print these traces until this gets overhauled.
+    is_xonsh_error = exc_info[0] in (XonshError, XonshCalledProcessError)
+
+    # hide unhelpful traces if not debugging
+    hide_stacktrace = debug_level == 0 and (is_syntax_error or is_xonsh_error)
+    if hide_stacktrace:
+        limit = 0
+        chain = False
+
+    sys.last_type, sys.last_value, sys.last_traceback = exc_info
+
     manually_set_trace, show_trace = _get_manual_env_var("XONSH_SHOW_TRACEBACK", False)
     manually_set_logfile, log_file = _get_manual_env_var("XONSH_TRACEBACK_LOGFILE")
     if (not manually_set_trace) and (not manually_set_logfile):
@@ -982,7 +1049,24 @@ def print_exception(msg=None):
                 "xonsh: To log full traceback to a file set: "
                 "$XONSH_TRACEBACK_LOGFILE = <filename>\n"
             )
-        traceback.print_exc()
+
+        traceback_str = "".join(
+            traceback.format_exception(*exc_info, limit=limit, chain=chain)
+        )
+
+        # color the traceback if available
+        _, interactive = _get_manual_env_var("XONSH_INTERACTIVE", 0)
+        _, color_results = _get_manual_env_var("COLOR_RESULTS", 0)
+        if interactive and color_results and HAS_PYGMENTS:
+            import pygments.lexers.python
+
+            lexer = pygments.lexers.python.PythonTracebackLexer()
+            tokens = list(pygments.lex(traceback_str, lexer=lexer))
+            # this goes to stdout, but since we are interactive it doesn't matter
+            print_color(tokens, end="")
+        else:
+            print(traceback_str, file=sys.stderr, end="")
+
     # additionally, check if a file for traceback logging has been
     # specified and convert to a proper option if needed
     log_file = to_logfile_opt(log_file)
@@ -990,22 +1074,22 @@ def print_exception(msg=None):
         # if log_file <> '' or log_file <> None, append
         # traceback log there as well
         with open(os.path.abspath(log_file), "a") as f:
-            traceback.print_exc(file=f)
+            traceback.print_exception(*exc_info, limit=limit, chain=chain, file=f)
 
     if not show_trace:
         # if traceback output is disabled, print the exception's
         # error message on stderr.
-        display_error_message()
+        display_error_message(exc_info)
     if msg:
         msg = msg if msg.endswith("\n") else msg + "\n"
         sys.stderr.write(msg)
 
 
-def display_error_message(strip_xonsh_error_types=True):
+def display_error_message(exc_info, strip_xonsh_error_types=True):
     """
-    Prints the error message of the current exception on stderr.
+    Prints the error message of the given sys.exc_info() triple on stderr.
     """
-    exc_type, exc_value, exc_traceback = sys.exc_info()
+    exc_type, exc_value, exc_traceback = exc_info
     exception_only = traceback.format_exception_only(exc_type, exc_value)
     if exc_type is XonshError and strip_xonsh_error_types:
         exception_only[0] = exception_only[0].partition(": ")[-1]
@@ -1207,8 +1291,8 @@ def always_none(x):
 
 
 def ensure_string(x):
-    """Returns a string if x is not a string, and x if it already is."""
-    return str(x)
+    """Returns a string if x is not a string, and x if it already is. If x is None, the empty string is returned."""
+    return str(x) if x is not None else ""
 
 
 def is_path(x):
@@ -1223,11 +1307,10 @@ def is_env_path(x):
 
 def str_to_path(x):
     """Converts a string to a path."""
-    if x is None:
+    if x is None or x == "":
         return None
     elif isinstance(x, str):
-        # checking x is needed to avoid uncontrolled converting empty string to Path('.')
-        return pathlib.Path(x) if x else None
+        return pathlib.Path(x)
     elif isinstance(x, pathlib.Path):
         return x
     elif isinstance(x, EnvPath) and len(x) == 1:
@@ -1248,7 +1331,7 @@ def str_to_env_path(x):
 
 def path_to_str(x):
     """Converts a path to a string."""
-    return str(x)
+    return str(x) if x is not None else ""
 
 
 def env_path_to_str(x):
@@ -1282,15 +1365,14 @@ def is_logfile_opt(x):
 
 
 def to_logfile_opt(x):
-    """
-    Converts a $XONSH_TRACEBACK_LOGFILE option to either a str containing
+    """Converts a $XONSH_TRACEBACK_LOGFILE option to either a str containing
     the filepath if it is a writable file or None if the filepath is not
     valid, informing the user on stderr about the invalid choice.
     """
-    if isinstance(x, os.PathLike):
+    if isinstance(x, os.PathLike):  # type: ignore
         x = str(x)
     if is_logfile_opt(x):
-        return x
+        return expand_path(x) if x else x
     else:
         # if option is not valid, return a proper
         # option and inform the user on stderr
@@ -1350,6 +1432,14 @@ def to_itself(x):
     return x
 
 
+def to_int_or_none(x) -> tp.Optional[int]:
+    """Convert the given value to integer if possible. Otherwise return None"""
+    if isinstance(x, str) and x.lower() == "none":
+        return None
+    else:
+        return int(x)
+
+
 def bool_to_str(x):
     """Converts a bool to an empty string if False and the string '1' if
     True.
@@ -1404,6 +1494,33 @@ def SLICE_REG():
     )
 
 
+def to_shlvl(x):
+    """Converts a value to an $SHLVL integer according to bash's behaviour (variables.c::adjust_shell_level)."""
+    if x is None:
+        return 0
+    else:
+        x = str(x)
+    try:
+        return adjust_shlvl(max(0, int(x)), 0)
+    except ValueError:
+        return 0
+
+
+def is_valid_shlvl(x):
+    """Checks whether a variable is a proper $SHLVL integer."""
+    return isinstance(x, int) and to_shlvl(x) == x
+
+
+def adjust_shlvl(old_lvl: int, change: int):
+    """Adjusts an $SHLVL integer according to bash's behaviour (variables.c::adjust_shell_level)."""
+    new_level = old_lvl + change
+    if new_level < 0:
+        new_level = 0
+    elif new_level >= 1000:
+        new_level = 1
+    return new_level
+
+
 def ensure_slice(x):
     """Try to convert an object into a slice, complain on failure"""
     if not x and x != 0:
@@ -1423,12 +1540,12 @@ def ensure_slice(x):
             groups = (int(i) if i else None for i in m.groups())
             s = slice(*groups)
         else:
-            raise ValueError("cannot convert {!r} to slice".format(x))
+            raise ValueError(f"cannot convert {x!r} to slice")
     except TypeError:
         try:
             s = slice(*(int(i) for i in x))
         except (TypeError, ValueError):
-            raise ValueError("cannot convert {!r} to slice".format(x))
+            raise ValueError(f"cannot convert {x!r} to slice")
     return s
 
 
@@ -1437,8 +1554,8 @@ def get_portions(it, slices):
 
     Parameters
     ----------
-    it: iterable
-    slices: a slice or a list of slice objects
+    it : iterable
+    slices : a slice or a list of slice objects
     """
     if is_slice(slices):
         slices = [slices]
@@ -1591,7 +1708,7 @@ def ptk2_color_depth_setter(x):
     elif x in {"", None}:
         x = ""
     else:
-        msg = '"{}" is not a valid value for $PROMPT_TOOLKIT_COLOR_DEPTH. '.format(x)
+        msg = f'"{x}" is not a valid value for $PROMPT_TOOLKIT_COLOR_DEPTH. '
         warnings.warn(msg, RuntimeWarning)
         x = ""
     if x == "" and "PROMPT_TOOLKIT_COLOR_DEPTH" in os_environ:
@@ -1602,10 +1719,12 @@ def ptk2_color_depth_setter(x):
 
 
 def is_completions_display_value(x):
+    """Enumerated values of ``$COMPLETIONS_DISPLAY``"""
     return x in {"none", "single", "multi"}
 
 
 def to_completions_display_value(x):
+    """Convert user input to value of ``$COMPLETIONS_DISPLAY``"""
     x = str(x).lower()
     if x in {"none", "false"}:
         x = "none"
@@ -1614,13 +1733,40 @@ def to_completions_display_value(x):
     elif x in {"single", "readline"}:
         pass
     else:
-        msg = '"{}" is not a valid value for $COMPLETIONS_DISPLAY. '.format(x)
+        msg = f'"{x}" is not a valid value for $COMPLETIONS_DISPLAY. '
         msg += 'Using "multi".'
         warnings.warn(msg, RuntimeWarning)
         x = "multi"
     return x
 
 
+CANONIC_COMPLETION_MODES = frozenset({"default", "menu-complete"})
+
+
+def is_completion_mode(x):
+    """Enumerated values of $COMPLETION_MODE"""
+    return x in CANONIC_COMPLETION_MODES
+
+
+def to_completion_mode(x):
+    """Convert user input to value of $COMPLETION_MODE"""
+    y = str(x).casefold().replace("_", "-")
+    y = (
+        "default"
+        if y in ("", "d", "xonsh", "none", "def")
+        else "menu-complete"
+        if y in ("m", "menu", "menu-completion")
+        else y
+    )
+    if y not in CANONIC_COMPLETION_MODES:
+        warnings.warn(
+            f"'{x}' is not valid for $COMPLETION_MODE, must be one of {CANONIC_COMPLETION_MODES}.  Using 'default'.",
+            RuntimeWarning,
+        )
+        y = "default"
+    return y
+
+
 def is_str_str_dict(x):
     """Tests if something is a str:str dictionary"""
     return isinstance(x, dict) and all(
@@ -1635,7 +1781,7 @@ def to_dict(x):
     try:
         x = ast.literal_eval(x)
     except (ValueError, SyntaxError):
-        msg = '"{}" can not be converted to Python dictionary.'.format(x)
+        msg = f'"{x}" can not be converted to Python dictionary.'
         warnings.warn(msg, RuntimeWarning)
         x = dict()
     return x
@@ -1647,7 +1793,7 @@ def to_str_str_dict(x):
         return x
     x = to_dict(x)
     if not is_str_str_dict(x):
-        msg = '"{}" can not be converted to str:str dictionary.'.format(x)
+        msg = f'"{x}" can not be converted to str:str dictionary.'
         warnings.warn(msg, RuntimeWarning)
         x = dict()
     return x
@@ -1670,7 +1816,7 @@ _year_to_sec = lambda x: 365.25 * _day_to_sec(x)
 _kb_to_b = lambda x: 1024 * int(x)
 _mb_to_b = lambda x: 1024 * _kb_to_b(x)
 _gb_to_b = lambda x: 1024 * _mb_to_b(x)
-_tb_to_b = lambda x: 1024 * _tb_to_b(x)
+_tb_to_b = lambda x: 1024 * _tb_to_b(x)  # type: ignore
 
 CANON_HISTORY_UNITS = LazyObject(
     lambda: frozenset(["commands", "files", "s", "b"]), globals(), "CANON_HISTORY_UNITS"
@@ -1808,7 +1954,7 @@ def to_history_tuple(x):
 
 def history_tuple_to_str(x):
     """Converts a valid history tuple to a canonical string."""
-    return "{0} {1}".format(*x)
+    return "{} {}".format(*x)
 
 
 def all_permutations(iterable):
@@ -1822,7 +1968,14 @@ def format_color(string, **kwargs):
     shell instances method of the same name. The results of this function should
     be directly usable by print_color().
     """
-    return builtins.__xonsh__.shell.shell.format_color(string, **kwargs)
+    if hasattr(xsh.shell, "shell"):
+        return xsh.shell.shell.format_color(string, **kwargs)
+    else:
+        # fallback for ANSI if shell is not yet initialized
+        from xonsh.ansi_colors import ansi_partial_color_format
+
+        style = xsh.env.get("XONSH_COLOR_STYLE")
+        return ansi_partial_color_format(string, style=style)
 
 
 def print_color(string, **kwargs):
@@ -1830,24 +1983,66 @@ def print_color(string, **kwargs):
     method of the same name. Colors will be formatted if they have not already
     been.
     """
-    builtins.__xonsh__.shell.shell.print_color(string, **kwargs)
+    if hasattr(xsh.shell, "shell"):
+        xsh.shell.shell.print_color(string, **kwargs)
+    else:
+        # fallback for ANSI if shell is not yet initialized
+        print(format_color(string, **kwargs))
 
 
 def color_style_names():
     """Returns an iterable of all available style names."""
-    return builtins.__xonsh__.shell.shell.color_style_names()
+    return xsh.shell.shell.color_style_names()
 
 
 def color_style():
     """Returns the current color map."""
-    return builtins.__xonsh__.shell.shell.color_style()
+    return xsh.shell.shell.color_style()
+
+
+def register_custom_style(
+    name, styles, highlight_color=None, background_color=None, base="default"
+):
+    """Register custom style.
+
+    Parameters
+    ----------
+    name : str
+        Style name.
+    styles : dict
+        Token -> style mapping.
+    highlight_color : str
+        Hightlight color.
+    background_color : str
+        Background color.
+    base : str, optional
+        Base style to use as default.
+
+    Returns
+    -------
+    style : The style object created, None if not succeeded
+    """
+    style = None
+    if pygments_version_info():
+        from xonsh.pyghooks import register_custom_pygments_style
+
+        style = register_custom_pygments_style(
+            name, styles, highlight_color, background_color, base
+        )
+
+    # register ANSI colors
+    from xonsh.ansi_colors import register_custom_ansi_style
+
+    register_custom_ansi_style(name, styles, base)
+
+    return style
 
 
 def _token_attr_from_stylemap(stylemap):
-    """yields tokens attr, and index from a stylemap """
+    """yields tokens attr, and index from a stylemap"""
     import prompt_toolkit as ptk
 
-    if builtins.__xonsh__.shell.shell_type == "prompt_toolkit1":
+    if xsh.shell.shell_type == "prompt_toolkit1":
         style = ptk.styles.style_from_dict(stylemap)
         for token in stylemap:
             yield token, style.token_to_attrs[token]
@@ -1861,8 +2056,8 @@ def _token_attr_from_stylemap(stylemap):
 
 
 def _get_color_lookup_table():
-    """Returns the prompt_toolkit win32 ColorLookupTable """
-    if builtins.__xonsh__.shell.shell_type == "prompt_toolkit1":
+    """Returns the prompt_toolkit win32 ColorLookupTable"""
+    if xsh.shell.shell_type == "prompt_toolkit1":
         from prompt_toolkit.terminal.win32_output import ColorLookupTable
     else:
         from prompt_toolkit.output.win32 import ColorLookupTable
@@ -1870,7 +2065,7 @@ def _get_color_lookup_table():
 
 
 def _get_color_indexes(style_map):
-    """Generates the color and windows color index for a style """
+    """Generates the color and windows color index for a style"""
     table = _get_color_lookup_table()
     for token, attr in _token_attr_from_stylemap(style_map):
         if attr.color:
@@ -1937,16 +2132,14 @@ def _win10_color_map():
         "ansibrightyellow": (249, 241, 165),
         "ansiwhite": (242, 242, 242),
     }
-    return {
-        k: "#{0:02x}{1:02x}{2:02x}".format(r, g, b) for k, (r, g, b) in cmap.items()
-    }
+    return {k: f"#{r:02x}{g:02x}{b:02x}" for k, (r, g, b) in cmap.items()}
 
 
 WIN10_COLOR_MAP = LazyObject(_win10_color_map, globals(), "WIN10_COLOR_MAP")
 
 
 def _win_bold_color_map():
-    """ Map dark ansi colors to lighter version. """
+    """Map dark ansi colors to lighter version."""
     return {
         "ansiblack": "ansibrightblack",
         "ansiblue": "ansibrightblue",
@@ -1967,8 +2160,8 @@ def hardcode_colors_for_win10(style_map):
     in conhost.exe
     """
     modified_style = {}
-    if not builtins.__xonsh__.env["PROMPT_TOOLKIT_COLOR_DEPTH"]:
-        builtins.__xonsh__.env["PROMPT_TOOLKIT_COLOR_DEPTH"] = "DEPTH_24_BIT"
+    if not xsh.env["PROMPT_TOOLKIT_COLOR_DEPTH"]:
+        xsh.env["PROMPT_TOOLKIT_COLOR_DEPTH"] = "DEPTH_24_BIT"
     # Replace all ansi colors with hardcoded colors to avoid unreadable defaults
     # in conhost.exe
     for token, style_str in style_map.items():
@@ -2015,7 +2208,7 @@ def intensify_colors_for_cmd_exe(style_map):
         6: "ansibrightyellow",  # subst yellow with bright yellow
         9: "ansicyan",  # subst intense blue with dark cyan (more readable)
     }
-    if builtins.__xonsh__.shell.shell_type == "prompt_toolkit1":
+    if xsh.shell.shell_type == "prompt_toolkit1":
         replace_colors = ansicolors_to_ptk1_names(replace_colors)
     for token, idx, _ in _get_color_indexes(style_map):
         if idx in replace_colors:
@@ -2028,12 +2221,8 @@ def intensify_colors_on_win_setter(enable):
     environment variable.
     """
     enable = to_bool(enable)
-    if (
-        hasattr(builtins.__xonsh__, "shell")
-        and builtins.__xonsh__.shell is not None
-        and hasattr(builtins.__xonsh__.shell.shell.styler, "style_name")
-    ):
-        delattr(builtins.__xonsh__.shell.shell.styler, "style_name")
+    if xsh.shell is not None and hasattr(xsh.shell.shell.styler, "style_name"):
+        delattr(xsh.shell.shell.styler, "style_name")
     return enable
 
 
@@ -2043,12 +2232,12 @@ def format_std_prepost(template, env=None):
     """
     if not template:
         return ""
-    env = builtins.__xonsh__.env if env is None else env
+    env = xsh.env if env is None else env
     invis = "\001\002"
-    if builtins.__xonsh__.shell is None:
+    if xsh.shell is None:
         # shell hasn't fully started up (probably still in xonshrc)
-        from xonsh.prompt.base import PromptFormatter
         from xonsh.ansi_colors import ansi_partial_color_format
+        from xonsh.prompt.base import PromptFormatter
 
         pf = PromptFormatter()
         s = pf(template)
@@ -2056,7 +2245,7 @@ def format_std_prepost(template, env=None):
         s = ansi_partial_color_format(invis + s + invis, hide=False, style=style)
     else:
         # shell has fully started. do the normal thing
-        shell = builtins.__xonsh__.shell.shell
+        shell = xsh.shell.shell
         try:
             s = shell.prompt_formatter(template)
         except Exception:
@@ -2231,7 +2420,7 @@ def POSIX_ENVVAR_REGEX():
 def expandvars(path):
     """Expand shell variables of the forms $var, ${var} and %var%.
     Unknown variables are left unchanged."""
-    env = builtins.__xonsh__.env
+    env = xsh.env
     if isinstance(path, bytes):
         path = path.decode(
             encoding=env.get("XONSH_ENCODING"), errors=env.get("XONSH_ENCODING_ERRORS")
@@ -2270,7 +2459,7 @@ def backup_file(fname):
 
     base, ext = os.path.splitext(fname)
     timestamp = datetime.now().strftime("%Y-%m-%d-%H-%M-%S-%f")
-    newfname = "%s.%s%s" % (base, timestamp, ext)
+    newfname = f"{base}.{timestamp}{ext}"
     shutil.move(fname, newfname)
 
 
@@ -2280,7 +2469,7 @@ def normabspath(p):
 
 
 def expanduser_abs_path(inp):
-    """ Provides user expanded absolute path """
+    """Provides user expanded absolute path"""
     return os.path.abspath(expanduser(inp))
 
 
@@ -2313,11 +2502,11 @@ def expand_case_matching(s):
         elif c.isalpha():
             folded = c.casefold()
             if len(folded) == 1:
-                c = "[{0}{1}]".format(c.upper(), c.lower())
+                c = f"[{c.upper()}{c.lower()}]"
             else:
-                newc = ["[{0}{1}]?".format(f.upper(), f.lower()) for f in folded[:-1]]
+                newc = [f"[{f.upper()}{f.lower()}]?" for f in folded[:-1]]
                 newc = "".join(newc)
-                newc += "[{0}{1}{2}]".format(folded[-1].upper(), folded[-1].lower(), c)
+                newc += f"[{folded[-1].upper()}{folded[-1].lower()}{c}]"
                 c = newc
         t.append(c)
     return "".join(t)
@@ -2352,11 +2541,11 @@ def _dotglobstr(s):
 
 
 def _iglobpath(s, ignore_case=False, sort_result=None, include_dotfiles=None):
-    s = builtins.__xonsh__.expand_path(s)
+    s = xsh.expand_path(s)
     if sort_result is None:
-        sort_result = builtins.__xonsh__.env.get("GLOB_SORTED")
+        sort_result = xsh.env.get("GLOB_SORTED")
     if include_dotfiles is None:
-        include_dotfiles = builtins.__xonsh__.env.get("DOTGLOB")
+        include_dotfiles = xsh.env.get("DOTGLOB")
     if ignore_case:
         s = expand_case_matching(s)
     if "**" in s and "**/*" not in s:
@@ -2398,7 +2587,7 @@ def ensure_timestamp(t, datetime_format=None):
     except (ValueError, TypeError):
         pass
     if datetime_format is None:
-        datetime_format = builtins.__xonsh__.env["XONSH_DATETIME_FORMAT"]
+        datetime_format = xsh.env["XONSH_DATETIME_FORMAT"]
     if isinstance(t, datetime.datetime):
         t = t.timestamp()
     else:
@@ -2408,7 +2597,7 @@ def ensure_timestamp(t, datetime_format=None):
 
 def format_datetime(dt):
     """Format datetime object to string base on $XONSH_DATETIME_FORMAT Env."""
-    format_ = builtins.__xonsh__.env["XONSH_DATETIME_FORMAT"]
+    format_ = xsh.env["XONSH_DATETIME_FORMAT"]
     return dt.strftime(format_)
 
 
@@ -2511,7 +2700,7 @@ def deprecated(deprecated_in=None, removed_in=None):
         message_suffix = ""
 
     def decorated(func):
-        warning_message = "{} has been deprecated".format(func.__name__)
+        warning_message = f"{func.__name__} has been deprecated"
         warning_message += message_suffix
 
         @functools.wraps(func)
@@ -2521,7 +2710,7 @@ def deprecated(deprecated_in=None, removed_in=None):
             warnings.warn(warning_message, DeprecationWarning)
 
         wrapped.__doc__ = (
-            "{}\n\n{}".format(wrapped.__doc__, warning_message)
+            f"{wrapped.__doc__}\n\n{warning_message}"
             if wrapped.__doc__
             else warning_message
         )
@@ -2537,9 +2726,9 @@ def _deprecated_message_suffix(deprecated_in, removed_in):
             deprecated_in, removed_in
         )
     elif deprecated_in and not removed_in:
-        message_suffix = " in version {}".format(deprecated_in)
+        message_suffix = f" in version {deprecated_in}"
     elif not deprecated_in and removed_in:
-        message_suffix = " and will be removed in version {}".format(removed_in)
+        message_suffix = f" and will be removed in version {removed_in}"
     else:
         message_suffix = None
 
@@ -2547,9 +2736,19 @@ def _deprecated_message_suffix(deprecated_in, removed_in):
 
 
 def _deprecated_error_on_expiration(name, removed_in):
+    from packaging.version import Version
+
     if not removed_in:
         return
-    elif LooseVersion(__version__) >= LooseVersion(removed_in):
-        raise AssertionError(
-            "{} has passed its version {} expiry date!".format(name, removed_in)
-        )
+    elif Version(__version__) >= Version(removed_in):
+        raise AssertionError(f"{name} has passed its version {removed_in} expiry date!")
+
+
+def to_repr_pretty_(inst, p, cycle):
+    name = f"{inst.__class__.__module__}.{inst.__class__.__name__}"
+    with p.group(0, name + "(", ")"):
+        if cycle:
+            p.text("...")
+        elif len(inst):
+            p.break_()
+            p.pretty(dict(inst))
